// chain.cpp: implementation of the chain class.
//
//////////////////////////////////////////////////////////////////////
#include "chain.h"
#include "missing.h"
#include <fstream>
using namespace::std;
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

chain::chain(imat &d,iragged ph,ctsdistribution *ap, double tempering)
	:variable(d),_miss(d),data(d),haps(data,1),hl(haps)
  ,_samples(d.rows()),_tempering(tempering),phase(ph)
{	
#ifdef CHECK
  assert(check("after construction")==true);
#endif
  if (ap->undefined()) {
    cout << "undefined alphaprior using uniform(0,1000)"<<endl;
    alphaprior = new uniform(0,1000);
  } else alphaprior=ap->clone();
  _alpha = alphaprior->sample();
}

//imat dat(0,0);
chain::chain(void):data(),_miss(),variable(),phase()
{
}


chain::~chain()
{
}
//
//
bool chain::update(rng &r)
{
  int which = variable[r.rint(1,variable())];	
  haplotype h1(haps.columns());haplotype h2(haps.columns());
  haplotype oh1(haps.columns());haplotype oh2(haps.columns());
  getnewphase(h1,h2,data[which],phase,r);
  haps.getold(which,oh1,oh2);

  for (int i=1;i<=haps.columns();i++) {
    if (h1[i]==oh1[i]) {
      if (h2[i]!=oh2[i]) {
	cout << "locus " << i <<" which "<<which<< endl;
	cout <<" new"<< h1 << " " << h2<< endl;
	cout << "old"<<oh1 << " "<<oh2<< endl;
	phase.print(cout);
	error e("error 1\n");
      }
     } else if (h1[i]==oh2[i]) {
	if (h2[i]!=oh1[i]) {
	  cout <<"new"<< h1 << " " << h2<< endl;
	  cout << "old"<<oh1 << " "<<oh2<< endl;
	  error e("error 2");
	}
     } else { 
	  cout <<"new"<< h1 << " " << h2<< endl;
	  cout << "old"<<oh1 << " "<<oh2<< endl;
	  error e("error 3");
	}
  } 

  int nf1 = hl[h1];
  int nf2 = hl[h2];

  int f1 = hl[oh1];
  int f2 = hl[oh2];

  if (h1==oh1||h1==oh2) return false;

  int diffn=0;		
  double diffll = 0.0;

  if (h1==h2) {
    if (nf1==0) diffn+=1;
    diffll += lgamma(nf1+2.0);
  } else {
    diffll += lgamma(nf1+1.0)+lgamma(nf2+1.0);
    diffn += (nf1==0)+(nf2==0);
  }
  if (oh1==oh2) {		
    assert(f1>1);
    diffll -= lgamma(f2-2.0);
    if (f1==2) diffn-=1;
  } else {
    diffll -= lgamma(f1+1.0)+lgamma(f2+1.0);
    diffn -= (f1==1)+(f2==1);
  }

  diffll += diffn*log(alpha());
  double x=exp(diffll/_tempering);
	
  if (x>1.0||r.next()<x)  {		
    hl.add(h1);
    hl.add(h2);
    hl.remove(oh1);
    hl.remove(oh2);
    haps.change(which,h1,h2);	
    return true;
  }
  return false;
}
//
//
//
bool chain::metromissing(rng &r)
{
   gpair g=_miss.rndmissing(r);
  int ogeno[2];
   ogeno[0] = data[g.individual][2*g.locus-1];
  ogeno[1] = data[g.individual][2*g.locus];
  haplotype oh1(haps.columns());
  haplotype oh2(haps.columns());
  haps.getold(g.individual,oh1,oh2);

  data[g.individual][2*g.locus-1] = r.rint(1,2);
  data[g.individual][2*g.locus] = r.rint(1,2);
  haplotype h1(haps.columns());
  haplotype h2(haps.columns());
  getnewphase(h1,h2,data[g.individual],phase,r);

  int nf1 = hl[h1];
  int nf2 = hl[h2];

  int f1 = hl[oh1];
  int f2 = hl[oh2];

  // if (h1==oh1||h1==oh2) return false;
  //	assert(h1!=oh1&&h1!=oh2&&h2!=oh1&&h2!=oh2);

  int diffn=0;		
  double diffll = 0.0;
  if (h1==h2) {
    if (nf1==0) diffn+=1;
    diffll += lgamma(nf1+2.0);
  } else {
    diffll += lgamma(nf1+1.0)+lgamma(nf2+1.0);
    diffn += (nf1==0)+(nf2==0);
  }
  if (oh1==oh2) {		
    assert(f1>1);
    diffll -= lgamma(f2-2.0);
    if (f1==2) diffn-=1;
  } else {
    diffll -= lgamma(f1+1.0)+lgamma(f2+1.0);
    diffn -= (f1==1)+(f2==1);
  }
	
  diffll += diffn*log(alpha());
  double x=exp(diffll/_tempering);
	
  if (x>1.0||r.next()<x)  {		
    hl.add(h1);
    hl.add(h2);
    hl.remove(oh1);
    hl.remove(oh2);
    haps.change(g.individual,h1,h2);	
#ifdef CHECK
       assert(check("missing success\n")==true);
#endif
    return true;
  } else {
    data[g.individual][2*g.locus-1] = ogeno[0];
    data[g.individual][2*g.locus]   = ogeno[1];
#ifdef CHECK
      assert(check("missing failure\n")==true);
#endif
    return false;
  }
}
//
//
//
void chain::printfreq(ostream &o)
{
	hl.print(o);
}
//
//
//
void chain::printgeno(ostream &o)
{
	o << data;
}
//
//
//
bool chain::updatealpha(rng &r)
{
  double newalpha=exp(log(alpha()) + r()-0.5);

  double diffll = (
		   n()*(log(newalpha)-log(_alpha))
		   +lgamma(newalpha)-lgamma(_alpha) 
		   -lgamma(haps.rows()+newalpha) + lgamma(haps.rows()+_alpha));

  double x=exp(log(newalpha)-log(alpha())+diffll/temper());
  if (x>1.0||r()<x)
    {
      _alpha=newalpha;
      return true;
    }
  return false;
}
//
//
//
int chain::updatehaps(int n, rng &r)
{
	int ret=0;
	for (int i =1;i<=n;i++) ret += update(r);
#ifdef CHECK
	check("after updatehaps");
#endif
	return ret;
}
//
//   
//
int chain::updatemissing(int n, rng &r)
{
  if (miss()) {
	int ret=0;
	for (int i =1;i<=n;i++) ret += metromissing(r);
	return ret;
  } else return 0;
}
//
//
//
double chain::posterior(void)
{
	return hl.posterior(alpha())/_tempering;
}
//
//
//
chain &chain::operator=(const chain &a) 
{
  if (this==&a) return *this;
  _tempering=a._tempering;
  data = a.data;
  haps=a.haps;
  hl=a.hl;
  _samples=a._samples;
  _alpha=a._alpha;
  variable=a.variable;
  alphaprior=a.alphaprior;
  _miss=a._miss;
  phase=a.phase;
   return *this;
}
//
//
//
int metro_temper(chain *c,int nchains,ivec &order,rng &r)
{
/*	int wh[2];
	sample2int(wh,0,nchains-1,r);*/
	int wh = r.rint(0,nchains-2);
	int swap1=order(wh);
	int swap2=order(wh+1);
	double opost=c[swap1].posterior()+c[swap2].posterior();
	double tmp=c[swap1].temper();
	c[swap1].temper()=c[swap2].temper();
	c[swap2].temper()=tmp;
	
	double x = exp(c[swap1].posterior() + c[swap2].posterior() - opost);
	if (x>1.0||r()<x) {
		double tmp=c[swap1].temper();
		c[swap1].temper()=c[swap2].temper();
		c[swap2].temper()=tmp;
		return 0;
	} else {
		order(wh)=swap2;
		order(wh+1)=swap1;
		return 1;
	}
}

//
//   var class
//

var::var(imat &g):ivec(g.rows())
{
	int count=0;
	int loci=g.columns()/2;
	for (int i=1;i<=_length;i++) {
		if (isvariable(g[i],loci)) _data[count++]=i;
	}
    _nvar=count;
}
//
//
//
var::var():ivec()
{
  _nvar=0;
}
//
//
//
var &var::operator=(const var &a)
{
	if (this==&a) return *this;	
	_nvar=a._nvar;
    if (a.length()!=_length) {
	  if (_length!=0) delete [] _data;
    _length=a.length();
    _data = new int[_length];	
  }
  for (int i=0;i<_nvar;i++) _data[i]=a._data[i];

    return *this;
}

bool chain::check(char *message)
{
  // first check that the haplotypes agree with the genotypes
  for (int i=0;i<n();i++) {
    for (int j=0;j<data.columns()/2;j++) {
      if (data(i)[2*j]==haps(2*i)[j]) {//data 1 == hap 1
	if (data(i)[2*j+1]!=haps(2*i+1)[j]) { // data 2 != hap 2
	  ofstream err("err.txt");
	  cout << message << " problem 1 with check individual "
	       <<i+1<<" locus "<<j+1<<endl;
	  cout << "data: " << data(i)[2*j]<<" "<<data(i)[2*j+1]
	       <<"    h1: "<<haps(2*i)[j]<<" h2: "<<haps(2*i+1)[j]<<endl;
	  return false;
	}
      } else if (data(i)[2*j]==haps(2*i+1)[j]) { //data 1 == hap2
        if (data(i)[2*j+1]!=haps(2*i)[j]) { // data 2 != hap 1
	  ofstream err("err.txt");
	  cout <<  message << " problem 2 with check individual "
	       <<i+1<<" locus "<<j+1<<endl;
	  cout << "data: " << data(i)[2*j]<<" "<<data(i)[2*j+1]
	       <<"    h1: "<<haps(2*i)[j]<<" h2: "<<haps(2*i+1)[j]<<endl;
	  return false;
	}
      } else {
	ofstream err("err.txt");
	cout << message << " problem 3 with check individual "
	     <<i+1<<" locus "<<j+1<<endl;
	cout << "data: " << data(i)[2*j]<<" "<<data(i)[2*j+1]
	     <<"    h1: "<<haps(2*i)[j]<<" h2: "<<haps(2*i+1)[j]<<endl;
	return false;
      }
    }
  }
  return true;
}

