//  get haplotypes from genotypes using gibbs
//  sampling or MCMC etc
//
//  copyright Ian Wilson 2001
#include <iostream>
#include <fstream>
#include "imat.h"
#include "ivec.h"
#include "random.h"
#include "haplotypes.h"
#include "haplist.h"
#include "chain.h"
#include "results.h"
#include "input.h"
#include "raggedi.h"
#include "factor.h"
using namespace::std;

int main(int argc, char *argv[])
{
  int gap=1000;
  if (argc<3||argc>4) {
    error e("incorrect input use: \nhapgen infile outfile <seed>\n");
  }
  input m(argv[1]);
  imat data(m.datafile);
  factor labels;
  if (m.labelfile) {
    factor d(m.labelfile);
    labels=d;
  }
  iragged phase;
  if (m.phasefile) {
    ivec ph(m.phasefile);
    if (ph.length()!=data.columns()/2)
      error e("phase has incorrect length\n");
    iragged p(ph);
    phase=p;
  } else {
    ivec ph(1,data.columns()/2,1);
    iragged p(ph);
    phase=p;
  }  
  
  if (labels.length()>0) {
    if (data.rows()!=labels.length()) {
      cerr << labels.length() << " labels and "
	   <<data.rows()<< " rows of data\n";
      error e("labels should have the same length as data");
    }
  }
  chain tmp(data,phase,m.alphaprior);
  ofstream out(argv[2]);
  if (argc==4) m.seed=atoi(argv[3]);
  globalrandom.set(m.seed);

  const char *err = m.check();
  if (err!=NULL) error e(err);

  cout << m;
	
  chain *c = new chain[m.chains];

  results res(tmp.samples());
	
   for (int ch=0;ch<m.chains;ch++) {
      c[ch]=tmp;
      c[ch].temper()=1.0 + ch/4.;
      c[ch].samplealpha();
   }
  
  ivec order(0,m.chains-1,1);
 
  int i,j;
   for (i=1;i<=m.burnin;i++) {
    for (int j=0;j<m.chains;j++) {
      c[j].updatehaps(gap,globalrandom);
      c[j].updatealpha(globalrandom);
      if (m.chains>1) metro_temper(c,m.chains,order);
      if (c[j].miss()) c[j].updatemissing(c[j].nmissing(),globalrandom);
    }
  }
  dvec success(m.chains,0.0);
  dvec success_missing(m.chains,0.0);
  dvec successalpha(m.chains,0.0);
  double success_swap=0.0;
  for (i=1;i<=m.samples;i++) {
    for (j=0;j<m.chains;j++) {
        success(order(j)) += c[j].updatehaps(gap,globalrandom);
        successalpha(order(j)) += c[j].updatealpha(globalrandom);	    
	if (m.chains>1) success_swap += metro_temper(c,m.chains,order);
        if (c[j].miss()) success_missing(order(j)) 
	  += c[j].updatemissing(c[j].nmissing(),globalrandom);
     }

    res.addresults(c[order(0)]);
    // for (j=0;j<m.chains;j++) {
    j=order(0);
      cout << c[j].temper() << " " 
	   << c[j].n() <<" " << c[j].alpha() << " "<<c[j].posterior()<< " ";
      //  }
    cout << endl;
  }
  if (labels.length()>0) 
    res.printreslabel(out,data,m.samples,labels);
  else
    res.printres(out,data,m.samples);
	
  success /= ((double)gap*(double)m.samples);
  cout << "haplotypes: "<< success;
  if (c[1].miss()) {
    success_missing /= ((double)c[1].nmissing()*(double)m.samples);
    cout <<"missing: " << success_missing;
  }
	
  successalpha /= (double)m.samples;
  cout << "alpha "<< successalpha;
  success_swap /=  (double)m.samples*(double)m.chains;
  cout << "swap: " << success_swap << endl;
	
  return 1;  
}






