// haplist.cpp: implementation of the haplist class.
//
/////////////////////////////////////////////////////////////////////
#include <map>
#include <cassert>
#include "haplotypes.h"
#include "haplist.h"
using namespace::std;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
haplist::haplist(haplotypes &ht)
{
	for (int i=0;i<ht.rows();i++) {
		haplotype a(ht(i),ht.columns());
		A[a] += 1;
	}
	n_=A.size();
}
//
//
//
int haplist::add(const haplotype &a)
{
	int c = A[a]+=1;
	if (c==1) n_++;
	return c;
}
//
//
//
haplotype haplist::maxfreq()
{
	hapfreq::const_iterator i,mx;
	int maxcount=0;
	for (i=A.begin();i!=A.end();++i) {
		if ((*i).second>maxcount) 
		{
			mx=i;
			maxcount=(*i).second;
		}
	}
	return (*mx).first;
}
//
//
//
dvec haplist::frequencies(haplotypes &h)
{
  dvec tmp(h.rows());
  for (int i=0;i<h.rows();i++) {
    haplotype a(h(i),h.columns());
    tmp(i)=A[a];
  }
  return tmp;
}
//
//
//
haplotype haplist::hap(int i)
{
  hapfreq::const_iterator it;
  for (it=A.begin();it!=A.end();++it) {
    if (i==0) return (*it).first;
    i--;
  }		
}
//
//
//	
int haplist::remove(const haplotype &r)
{
	int c=A[r];
	assert(c>0);
	if (c==1) {
		A.erase(r);
		n_--;
	} else A[r] -=1;
	return c-1;
}

haplist::~haplist()
{
}
//
//
void haplist::print(ostream &o, int replicates)
{
  hapfreq::const_iterator i;
  for (i=A.begin();i!=A.end();++i) {
    if ((*i).second>0) {
      if (replicates==0) o <<  (*i).second;
      else o << (*i).second/(double)replicates;
      o << " " <<  (*i).first << endl;
    }
  }
  o<<endl;
}

int haplist::calcn(void)
{
	hapfreq::const_iterator i;
	int count=0;
	for (i=A.begin();i!=A.end();++i) if ((*i).second>0) count++;
	return count;
}

double haplist::posterior(double alpha)
{
    hapfreq::const_iterator i;
	double ll=0.0;
	int counts=0,countn=0;
	for (i=A.begin();i!=A.end();++i) if ((*i).second>0) {
		ll += lgamma((*i).second);
		counts += (*i).second;
		countn += 1;
	}
	return lgamma(alpha)-lgamma(counts+alpha)+ll+countn*log(alpha);
}
//
//
//
haplist &haplist::operator=(const haplist &a)
{
	if (this==&a) return *this;
	n_=a.n_;
	A=a.A;
    return *this;
}
