// haplotypes.cpp: implementation of the haplotypes class.
//
//////////////////////////////////////////////////////////////////////
#include <cassert>
#include "haplotypes.h"
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
haplotypes::haplotypes():imat()
{

}
//
haplotypes &haplotypes::operator=(const haplotypes &a)
{
	if (this != &a) {
      imat::operator=(a);   // call the base class assignment operator 
   }
   return *this;
}

//
haplotypes::~haplotypes()
{
}
/////////////////////////////////////////////////////////////
//
// get a random haplotype object from a genotype
/////////////////////////////////////////////////////////////
haplotypes::haplotypes(imat &data,rng &r)
:imat(data.rows()*2,data.columns()/2)
{
	int i,j,count=1;
	for (i=1;i<=data.rows();i++) {
		for (j=1;j<=data.columns()/2 ;j++) {
			int wh = r.rint(0,1);
			(*this)[count][j] = data[i][2*j-wh];
			(*this)[count+1][j] = data[i][2*j-1+wh];
		}
		count +=2;
	}
}
//
// I don't understand what this function is for
//
haplotypes::haplotypes(imat &data,int extremal)
:imat(data.rows()*2,data.columns()/2)
{
	int i,j,count=1;
	for (i=1;i<=data.rows();i++) {
		for (j=1;j<data.columns()/2 ;j++) {
			(*this)[count][j] = data[i][2*j-extremal];
			(*this)[count+1][j] = data[i][2*j-1+extremal];
		}
				(*this)[count][j] = data[i][2*j-extremal]; 
// otherwise both extremes would be identical
		(*this)[count+1][j] = data[i][2*j-1+extremal];
		count +=2;
	}
}
//
haplotypes::haplotypes(ivec &data,rng &r)
:imat(2,data.length()/2)
{
	int j;
	for (j=1;j<=data.length()/2 ;j++) {
		int wh = r.rint(0,1);
		(*this)[1][j] = data[2*j-wh];
		(*this)[2][j] = data[2*j-1+wh];
	}
}
//
void haplotypes::getold(int which, haplotype &h1, haplotype &h2)
{
	int j;
	for (j=1;j<=columns();j++) {
		h1[j]=(*this)[2*which-1][j];
		h2[j]=(*this)[2*which][j];
	}
}
//
void getnew(haplotype &h1, haplotype &h2, int  *data, rng &r)
{
  int j;

  for (j=0;j<h1.length() ;j++) {
    int wh = r.rint(0,1);
    h1(j)= data[2*j+1+wh];	
    h2(j) = data[2*j+2-wh];	
  }
}
//
void getnewphase(haplotype &h1, haplotype &h2, 
		 int  *data, iragged &phase, rng &r)
{
  int j;
  for (j=1;j<=phase.rows();j++) {
    int wh = r.rint(0,1);
    for (int k=1;k<=phase.cols(j);k++) {
      h1[phase.elem(j,k)]= data[2*phase.elem(j,k)-1+wh];	
      h2[phase.elem(j,k)] = data[2*phase.elem(j,k)-wh];	
    }
  }
}
//
haplotype::haplotype(haplotype &other,int *x):ivec(other.length())
{
  for (int i=0;i<other.length();i++) {
    if (x[2*i]==x[2*i+1]) _data[i]=x[2*i];
    else if (x[2*i]==other(i)) _data[i]=x[2*i+1];
    else {
      assert(x[2*i+1]==other(i));
      _data[i]=x[2*i];
    }
  }
}
//
haplotype::~haplotype()
{
}
//
//
ostream& operator <<(ostream &o,const haplotype &d)
{
	for (int i=1;i<=d.length();i++) o << d[i];
	//	o << endl;
	return o;
}
////////////////////////////////////////////////////////////
bool operator<(const haplotype &lhs,const  haplotype &rhs)
{
	assert(lhs.length()==rhs.length());
	int i;
	for (i=1;i<=lhs.length();i++) {
	  if (lhs[i]<rhs[i]) return 1;
	  if (lhs[i]>rhs[i]) return 0;
	}
	return 0;
}
////////////////////////////////////////////////////////////
bool operator<=(const haplotype &lhs,const  haplotype &rhs)
{
	assert(lhs.length()==rhs.length());
	int i;
	for (i=1;i<=lhs.length();i++) {
	  if (lhs[i]<rhs[i]) return 1;
	  if (lhs[i]>rhs[i]) return 0;
	}
    return 1;
}
////////////////////////////////////////////////////////////
bool operator>(const haplotype &a, const haplotype &b)
{
	assert(a.length()==b.length());
	for (int i=1;i<=a.length();i++) {
		if (a[i]>b[i]) return 1;
		if (a[i]<b[i]) return 0;
	}
	return 0;
}
////////////////////////////////////////////////////////////////////
bool operator>=(const haplotype &a, const haplotype &b)
{
	assert(a.length()==b.length());
	for (int i=1;i<=a.length();i++) {
		if (a[i]>b[i]) return 1;
		if (a[i]<b[i]) return 0;
	}
	return 1;
}
////////////////////////////////////////////////////////////////////
bool operator==(const haplotype &a, const haplotype &b)
{
	assert(a.length()==b.length());
	for (int i=1;i<=a.length();i++) {
		if (a[i]!=b[i]) return 0;
	}
	return 1;
}
////////////////////////////////////////////////////////////////////
bool operator!=(const haplotype &a, const haplotype &b)
{
	assert(a.length()==b.length());
	for (int i=1;i<=a.length();i++) {
		if (a[i]!=b[i]) return 1;
	}
	return 0;
}
/////////////////////////////////////////////////////////////////////
void haplotypes::change(int which, haplotype &h1, haplotype &h2)
{
	int j;
	for (j=1;j<=columns();j++) {
		(*this)[2*which-1][j]=h1[j];
		(*this)[2*which][j]=h2[j];
	}
}
/////////////////////////////////////////////////////////////////////
bool isvariable(int *data, int loci)
// to get a variable location you need two variable loci (or missing loci)
{
	int i,count=0;
	for ( i=1;i<=loci;i++) {
		if (data[2*i]!=data[2*i-1]) count++;
		else if (data[2*i]<0||data[2*i-1]<0) count++;
	}
	if (count >=2) return 1;
    else return 0;
}
////////////////////////////////////////////////////////////////////
/*haplotypes allhaps(int *data,int loci)
{
  int i,count=0;
  haplotypes h(
  for (i=1;i<=loci;i++) {
    if (data[2*i]!=data[2*i-1]) count++;
    else if (data[2*i]<0||data[2*i-1]<0) count++;
  }
}*/

















