#include "results.h"
#include "chain.h"
#include "haplist.h"
//
//  First constructors and destructor
//
results::results():_nhap(0)
{
}
//
results::results(int n):_nhap(n)
{
  res = new  haplist[n];
}
//
results::~results()
{
  delete [] res;
  _nhap=0;
}
//
//  Assignment operator
//
results &results::operator=(const results &a)
{
  if (this==&a) return *this;
  if (_nhap != a._nhap) {
    delete [] res;
    res = new haplist[a._nhap];
    _nhap=a._nhap;
  }
  for (int i=0;i<_nhap;i++) res[i]=a.res[i];
  return *this;
}
//
//  Add the haplotypes from a chain to the results.
//
void results::addresults(chain &c)
{
  haplotype h1(c.haps.columns());
  haplotype h2(c.haps.columns());
  for (int i=1;i<=c.samples();i++) {
    c.haps.getold(i,h1,h2);	
    if (h1<h2) res[i-1].add(h1);
    else res[i-1].add(h2);
  }	
}
//
//   Get the best results...
//
void results::bestoverall(haplist &hl, imat &d)
{
  for (int i=1;i<=_nhap;i++) {
    haplotype hap1 = res[i-1].maxfreq();
    hl.add(hap1);
    haplotype hap2(hap1,d(i-1));
    hl.add(hap2);
  }
}
//
//  Print results when we do not have labels
//
void results::printres(ostream &o,  imat &d, int replicates)
{
  haplist overall;
  bestoverall(overall,d);
  o << "overall:"<<endl;
  overall.print(o,false);
  for (int j=1;j<=_nhap;j++) {
    o << "individual:" << j << endl; 
    res[j-1].print(o,replicates);
  }
}
//
//  Print results with labelling
//
void results::printreslabel(ostream &o,  imat &d, 
			    int replicates,factor &label)
{
  haplist overall;
  haplist *labelhaplist = new haplist[label.length()];
  for (int i=1;i<=_nhap;i++) {
    haplotype hap1 = res[i-1].maxfreq();
    labelhaplist[label[i]].add(hap1);
    overall.add(hap1);
    haplotype hap2(hap1,d(i-1));
    labelhaplist[label[i]].add(hap2);
    overall.add(hap2);
  }
  o << "haplotype overall";
  int j;
  for (j=0;j<label.classes();j++) {
    o << " "<<label.key(j);
  }
  o << endl; 
  for (j =0;j< overall.n();j++){
    haplotype  h1=overall.hap(j);
    o << h1 << " " << overall[h1];
    for (int j=0;j<label.classes();j++) {
      o << " " << labelhaplist[j][h1];
    }
    o << endl;
  }
  o << "Individual Results"<<endl;
  for (j=0;j<_nhap;j++) {
    o << "individual:" << j << "  group " << label.key(label(j))<<endl; 
    res[j].print(o,replicates);
  }
}
//
//
//






