// dmat.cpp: implementation of the imat class.
//
//////////////////////////////////////////////////////////////////////
#include "dmat.h"
#include "dvec.h"
#include "myio.h"
#include "util.h"
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

void dmat::allocate(int rows,int columns)
{
  _rows=rows;
	_cols=columns;
	_data=new double[rows*columns];
}

dmat::dmat(int rows, int columns)
{
  allocate(rows,columns);
}

dmat::dmat(int rows, int columns, double init)
{
  allocate(rows,columns);
  for (int i=0;i<rows*columns;i++) _data[i]=init;
}
dmat::~dmat()
{
	delete[] _data;
	_rows=_cols=0;
}


dmat::dmat(const char *filename,bool byrow)
{
	ifstream in(filename);
	if (in==0) error e("error opening input file for imat");
	int columns;
	vector <double> a = readdoublemat(in,columns);
	if (a.size()%columns!=0) {
		cerr << "length of vector, " << a.size()
			<< "is not equal to rows*columns";
		error e("exiting");
	}
	int rows=a.size()/columns;
	allocate(rows,columns);
if (byrow) {
		for (int i=0;i<a.size();i++) _data[i]=a[i];	
} else {
		int count=0;
		for (int i=0;i<columns;i++) {
			for (int j=0;j<rows;j++) {
				*(_data +j*rows+i)=a[count++];

			}
		}
	}
}

dmat::dmat(dvec d, int rows, int columns, bool byrow)
{
	if (d.length()!=rows*columns) {
		cerr << "length of dvec, " << d.length()
			<< "is not equal to rows*columns";
		exit(0);
	}
	allocate(rows,columns);
	
	if (byrow) {
		for (int i=0;i<d.length();i++) _data[i]=d[i+1];	
	} else {
		int count=1;
		for (int i=0;i<columns;i++) {
			for (int j=0;j<rows;j++) {
				*(_data +j*rows+i)=d[count++];
			}
		}
	}	
}
//
//  indexing operators 
//
double & dmat::operator ()(int i, int j)
// indexed from 0 to n-1, 
{
	assert(i>=0);assert(i<_rows);
	assert(j>=0);assert(j<_cols);
	return *(_data + i*_cols+j);
}
double *dmat::operator ()(int i)
//indexed from 0 to n-1
{
	assert(i>=0&&i<_rows);
	return _data+i*_cols;
}
double *dmat::operator [](int i)
{
	assert(i>0);
	assert(i<=_rows);
	return _data+(i-1)*_cols-1;
}
//
double &dmat::val(int i, int j)
{
	assert(i>0);assert(i<=_rows);
	assert(j>0);assert(j<=_cols);
	return *(_data+(i-1)*_cols +j-1);
}

//
void dmat::print(ostream &o)
{
	for (int i=0;i<_rows;i++) {
		for (int j=0;j<_cols;j++) {
			o<<*(_data+(i*_cols)+j)<<" ";
		}
		o<< endl;
	}
}

ostream &operator<<(ostream &o, dmat &x)
{
	x.print(o);
	return o;
}
   
