// dvec.cpp: implementation of the dvec class.
//
//////////////////////////////////////////////////////////////////////
#include <vector>
#include "myio.h"
#include "dvec.h"
#include "ctsdistribution.h"
using namespace::std;
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

dvec::~dvec()
{
	delete[] _data;
	_length=0;
}

dvec::dvec(int len)
{
	assert(len>0);
	_data=new double[len];
	_length=len;
}

dvec::dvec(int len, const double init)
{
	_data=new double[len];
	_length=len;
     for (int i=0;i<len;i++) _data[i]=init;
}

int dvec::length()
{
	return _length;
}


void dvec::reallocate(int len, double init)
{
	if (len!=_length) {
		if (_length>0) delete [] _data;
		_data = new double[len];
		_length=len;
	}
	for (int i=0;i<len;i++) _data[i]=init;
}

double & dvec::operator ()(int i)
{
	assert(i>=0&&i<_length);
	return *(_data+i);
}
 
double & dvec::operator [](int i)
{
	 assert(i>=1&&i<=_length);
	 return *(_data+i-1);
}

double & dvec::operator [](int i) const
{
	assert(i>=1&&i<=_length);
	return *(_data+i-1);
}


void dvec::operator *=(const double &a)
{
	for (int i=0;i<_length;i++) _data[i]*=a;
}

void dvec::operator /=(const double &a)
{
	for (int i=0;i<_length;i++) _data[i]/=a;
}


void dvec::operator *=(const dvec &a)
{
	for (int i=0;i<_length;i++) _data[i]*=a._data[i];
}

dvec::dvec(ctsdistribution &d, int n)
{
  //	cerr << "constructor 3\n"; 
	assert(n>0);
	_data=new double[n];
	_length=n;
	for (int i=0;i<n;i++) _data[i]=d.sample();
}

dvec::dvec(const dvec &d)
{
  //	cerr << "constructor 2\n"; 
	_data=new double[d._length];
	_length=d._length;
	for (int i=0;i<d._length;i++) _data[i]=d._data[i];
}

ostream& operator <<(ostream &o,const dvec &d)
{
	for (int i=1;i<=d.length();i++) {
		o << d[i] << " ";
		//	if (i%5==0) o << endl;
	}
	o << endl;
	return o;
}

dvec operator+(const dvec &a, const dvec &b) 
{
	dvec tmp(a);
	for (int i=1;i<=a.length();i++) tmp[i]+=b[i];
	return tmp;
}

dvec operator*(const dvec &a, const dvec &b) 
{
	dvec tmp(a);
	for (int i=1;i<=a.length();i++) tmp[i]*=b[i];
	return tmp;
}

dvec operator/(const dvec &a, const dvec &b) 
{
	dvec tmp(a);
	for (int i=1;i<=a.length();i++) tmp[i]/=b[i];
	return tmp;
}

const dvec &dvec::operator=(const dvec &a) 
{
	if (this==&a) return *this;
	if (a.length()!=_length) {
		if (_length!=0) delete [] _data;
		_length=a.length();
		_data = new double[_length];
		
	}

	for (int i=0;i<a.length();i++) _data[i]=a._data[i];
	return *this;
}

double dvec::sum()
{
	double tmp=0.0;
	for (int i=0;i<_length;i++) tmp+=_data[i];
	return tmp;

}

double dvec::sum(int n)
{
	double tmp=0.0;
	for (int i=0;i<n;i++) tmp+=_data[i];
	return tmp;
}


double dvec::mean()
{
	return sum()/_length;
}
//
//
//
void dvec::avevar(double &ave, double &var)
{
	double s;
     
	ave=0.0;var=0.0;
	int j;
	for (j=0;j<_length;j++) ave += _data[j];
	ave /= _length;
	double ep(0.0);
	for (j=0;j<_length;j++) {
		s=_data[j] - ave;
		ep +=s;
		var += s*s;
	}
	var = (var-ep*ep/length())/(length()-1);
}
//
//  function to sort a vector
//
void dvec::sort()
{
    hpsort(_length,_data-1);
}
//
// function to difference a vector
//
void dvec::diff(double first)
{
  for (int i=_length-1;i>0;i--) _data[i]-=_data[i-1];
  _data[0]-=first;
}
//
//   Now for some IO
//
dvec readdoubles(istream &in, char until)
{
	vector <double> data = readdoublevector(in,until);
	dvec tmp(data.size());
	for (int i=1;i<=tmp.length();i++) tmp[i]=data[i-1];
    return tmp;
}
//
//  reads a number of doubles from a bracket
//  writes to a dvec object of any length
//
dvec get_doublesfrombrackets(istream &in) 
{  
	int ch=skipspace(in);
	if (ch!='(') {
		error e("need an opening brace");
	}	 
	double x;	
	vector <double> data;
	for (;;) {
	  in >> x;
	  data.push_back(x);
	  ch = skipspace(in);
	  if (ch==')') break;
	  if (ch!=',') error e("error should be comma");
	}	
	dvec tmp(data.size());
	for (int i=1;i<=tmp.length();i++) tmp[i]=data[i-1];
	return tmp;
}
//
dvec scan(istream &in, const char *namestring, dvec &default_val)
{ 
    if (findparametername(in,namestring)) {
	     return readdoubles(in);
    } else {
	cout << namestring 
	     << " not found in parameter file, using "
	     << default_val << endl;
	dvec x(default_val);
	return x;
    }
}
//
istream &operator>>(istream &in, dvec &a)
{
	a = readdoubles(in,'\n');
	return in;
}
