// imat.cpp: implementation of the imat class.
//
//////////////////////////////////////////////////////////////////////
#include <fstream>
#include <vector>
#include "util.h"
#include "myio.h"
#include "imat.h"
#include "ivec.h"
using namespace std;
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
////////////////////////////////////////////////////////////////////

imat::imat(int rows, int columns)
{
	allocate(rows,columns);
}

imat::imat(char *filename,bool usechar, bool byrow)
{
	ifstream in(filename);
	if (in==0) error e("error opening input file for imat");
	int columns;
	vector <int> a = readcharintmat(in,columns,usechar);
	if (a.size()%columns!=0) {
		cerr << "length of vector, " << a.size()
			<< "is not equal to rows*columns";
		error e("exiting");
	}
	int rows=a.size()/columns;
	allocate(rows,columns);
if (byrow) {
		for (int i=0;i<a.size();i++) _data[i]=a[i];	
} else {
		int count=0;
		for (int i=0;i<columns;i++) {
			for (int j=0;j<rows;j++) {
				*(_data +j*rows+i)=a[count++];

			}
		}
	}
}
//
//
//
imat::imat(const imat &a)
{
	allocate(a._rows,a._cols);
    for (int i=0;i<_rows*_cols;i++) _data[i]=a._data[i];	
}
//
//
//
imat::~imat()
{
	delete[] _data;
	_rows=_cols=0;
}
//
//
//
void imat::allocate(int rows, int columns)
{
	_rows=rows;
	_cols=columns;
	_data=new int[rows*columns];
//	for (int i=1;i<rows;i++) _data[i]=_data[i-1]+columns;
}
void imat::resize(int r,int c)
{

  if (r!=_rows||c!=_cols) {
	  if (_rows*_cols!=0) delete [] _data;
	  allocate(r,c);	
  }
}
imat::imat(ivec d, int rows, int columns, bool byrow)
{
	if (d.length()!=rows*columns) {
	cerr << "length of dvec, " << d.length()
		<< "is not equal to rows*columns";
	exit(0);
	}
	allocate(rows,columns);

	if (byrow) {
		for (int i=0;i<d.length();i++) _data[i]=d[i+1];	
	} else {
		int count=1;
		for (int i=0;i<columns;i++) {
			for (int j=0;j<rows;j++) {
				*(_data +j*rows+i)=d[count++];
			}
		}
	}	
}
imat::imat(vector <int> &a,int columns, bool byrow)
{
	if (a.size()%columns!=0) 
		cerr << "length of vector, " << a.size()
		<< "is not equal to rows*columns";
	exit(0);
	
	int rows=a.size()/columns;
	allocate(rows,columns);

	if (byrow) {
		for (int i=0;i<a.size();i++) _data[i]=a[i+1];	
	} else {
		int count=1;
		for (int i=0;i<columns;i++) {
			for (int j=0;j<rows;j++) {
				*(_data +j*rows+i)=a[count++];
			}
		}
	}	
}
//
//
//
int &imat::operator ()(int i, int j)
{
	assert(i>0&&i<=_rows);
	assert(j>0&&j<=_cols);
	return *(_data + (i-1)*_cols+j-1);
}
//
int *imat::operator[](int i) 
{ 
	assert(i>0);assert(i<=_rows);
	return _data+(i-1)*_cols -1 ;
}
//    
int *imat::operator()(int i) 
{ 
	assert(i>=0&&i<_rows);
	return _data+i*_cols;
}
//
imat &imat::operator=(const imat &a) 
{
  if (this==&a) return *this;
  resize(a._rows,a._cols);
  for (int i=0;i<_rows*_cols;i++) _data[i]=a._data[i];
  return *this;
}
//
//
//
int imat::rowsum(int i)
{
	int sm=0;
	int *d = _data+((i-1)*_cols);
	for (int j=0;j<_cols;j++) sm += d[j];
	return sm;
}
ivec imat::rowsum()
{
	ivec sm(_cols,0);

	for (int i=1;i<=_rows;i++) {
		int *d=_data+(i-1)*_cols;
		for (int j=0;j<_cols;j++) 	sm[i] += d[j];
	}
	return sm;
}
//
//
//
int imat::colsum(int j)
{
	int sm=0;
	for (int i=0;i<_cols;i++) sm += *_data*i+j-1;
	return sm;
}
ivec imat::colsum()
{
	ivec sm(_rows,0);
	for (int i=0;i<_cols;i++) {
		int *d=_data+i*_cols;
		for (int j=0;j<_rows;j++) {
			sm(j) += d[j];
		}
	}
	return sm;
}
//
//
//
imat imat::subset(ivec &userows, ivec &usecolumns)
{
	int nr = userows.length()>0?userows.length():rows();
	int nc = usecolumns.length()>0?usecolumns.length():columns();
	
	if (nr>rows()||nc>columns())
		error e("error: cannot subset integer matrix");
    imat m(nr,nc);
	
	if (nr==rows()&&nc==columns()) {
		m = *this;
		return m;
	}
	
	if (nr<rows()&&nc<columns()) {
		for (int i=0;i<nr;i++) {
			for (int j=0;j<nc;j++) m(i+1,j+1) = *(_data+(_cols*(userows(i)-1))+usecolumns(j)-1);
		}
	} else if (nr==rows()) {
		for (int i=0;i<rows();i++) {
			for (int j=0;j<usecolumns.length();j++) m(i+1,j+1) = *(_data+(_cols*i)+usecolumns(j)-1);
		}
	} else {	
		for (int i=0;i<userows.length();i++) {
			for (int j=0;j<columns();j++) m(i+1,j+1) = *(_data+(_cols*(userows(i)-1))+j);
		}
	}
	return m;
}
//
void imat::print(ostream &o)
{
	for (int i=0;i<_rows;i++) {
		for (int j=0;j<_cols;j++) {
			o<<*(_data+(i*_cols)+j)<<" ";
		}
		o<< endl;
	}
}

ostream &operator<<(ostream &o, imat &x)
{
	x.print(o);
	return o;
}
   
