// ivec.cpp: implementation of the ivec class.
//
//////////////////////////////////////////////////////////////////////
#include <vector>
#include "ivec.h"
#include "myio.h"
#include "random.h"
using namespace::std;
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
ivec::~ivec()
{
  //	cerr << "destructor\n";
	delete[] _data;
	_length=0;
}
     
void ivec::reallocate(int len)
{
    if (_data!=0) delete[] _data;
	_data=new int[len];
	_length=len;
}
//
void ivec::allocate(int len)
{
	_data=new int[len];
	_length=len;
}
//
void ivec::series(int begin, int end) 
{
  int j;
  if (_length!=end-begin+1) cerr << "wrong elements in series";
  for (j=0;j<=end-begin;j++) _data[j]=begin+j;
}

int & ivec::operator ()(int i)
{
	assert(i>=0);
	assert(i<length());
	return *(_data+i);
}
 
int & ivec::operator [](int i)
{
		assert(i>0);
	assert(i<=length());
	 return *(_data+i-1);
}

int & ivec::operator [](int i) const
{
	assert(i>=1&&i<=_length);
	 return *(_data+i-1);
}

void ivec::operator *=(const int &a)
{
	for (int i=0;i<_length;i++) _data[i]*=a;
}

void ivec::operator *=(const ivec &a)
{
	for (int i=0;i<_length;i++) _data[i]*=a._data[i];
}

ivec::ivec(const ivec &d)
{
//	cerr << "ivec ctor \n";
	allocate(d._length);
	for (int i=0;i<d._length;i++) _data[i]=d._data[i];
}

ivec::ivec(ivec &x, int n)
{
	assert(n<=x.length());
	allocate(n);
	for (int i=0;i<n;i++) _data[i]=x._data[i];
}

ivec::ivec(int from, int to ,int step)
// constructor for a series
{
	assert((from-to)%step==0);
	int n = (to-from)/step + 1;
	allocate(n);
	for (int i=0;i<n;i++) _data[i]=from+i*step;
}
//
ivec::ivec(const char *filename)
{
	ifstream in(filename);
	if (in==0) error e("error opening file");
	vector <int> x=readintvector(in,true,EOF);
	assert(x.size()>0);
	allocate(x.size());
	for (int i=0;i<_length;i++) _data[i]=x[i];
}
//
ostream& operator <<(ostream &o,const ivec &d)
{
	for (int i=1;i<=d.length();i++) {
		o << d[i] << " ";
		//	if (i%20==0) o << endl;
	}
	o << endl;
	return o;
}
ivec operator+(const ivec &a, const ivec &b) 
{
	ivec tmp(a);
	for (int i=1;i<=a.length();i++) tmp[i]+=b[i];
	return tmp;
}

ivec operator*(const ivec &a, const int b)
{
  ivec tmp(a);
	for (int i=1;i<=a.length();i++) tmp[i]*=b;
	return tmp;
}

ivec operator*(const ivec &a, const ivec &b) 
{
  assert(a.length()==b.length());
	ivec tmp(a);
	for (int i=1;i<=a.length();i++) tmp[i]*=b[i];
	return tmp;
}

ivec operator/(const ivec &a, const ivec &b) 
{
	ivec tmp(a);
	for (int i=1;i<=a.length();i++) tmp[i]/=b[i];
	return tmp;
}
const ivec &ivec::operator=(const ivec &a) 
{
  if (this==&a) return *this;
  if (a.length()!=_length) {
	  if (_length!=0) delete [] _data;
    _length=a.length();
    _data = new int[_length];	
  }
  for (int i=0;i<a.length();i++) _data[i]=a._data[i];
  return *this;
}

double ivec::sum()
{
	double tmp=0.0;
	for (int i=0;i<_length;i++) tmp+=_data[i];
	return tmp;

}
//
int ivec::isum()
{
	int tmp=0;
	for (int i=0;i<_length;i++) tmp+=_data[i];
	return tmp;
}
//
double ivec::mean()
{
	return sum()/_length;
}

int ivec::max()
{
	int mx=_data[1];
	for (int i=1;i<_length;i++) 
		if (_data[i]>mx) mx=_data[i];
	return mx;
}
//
//
ivec scan(istream &in, char *namestring, ivec &default_val)
{ 
    if (findparametername(in,namestring)) {
		ivec tmp;
		in >> tmp;
		return tmp;
    } else {
	cout << namestring 
	     << " not found in parameter file, using "
	     << default_val << endl;
	ivec x(default_val);
	return x;
    }
}
//
//
istream &operator>>(istream &in, ivec &a)
{
	vector <int> data=readintvector(in);
	a.reallocate(data.size());
	for (int i=1;i<=a.length();i++) a[i]=data[i-1];
    return in;
}
//
ivec ivec::rep(int rep_els, int rep_vec)
{
	ivec tmp(length()*rep_els*rep_vec);
	int count(0);
	for (int i=0;i<rep_vec;i++) {
		for (int j=0;j<length();j++) {
			for (int k=0;k<rep_els;k++) {
				tmp(count++) = j;
			}
		}
	}
	assert(count==tmp.length());
	return tmp;
}
//
void ivec::count(const ivec &c) 
{
  for (int i=0;i<length();i++) _data[i]=0;
  for (int j=1;j<=c.length();j++) {
    _data[c[j]-1]+=1;
  }
}
//
void ivec::count0(const ivec &c) 
{
  for (int i=0;i<length();i++) _data[i]=0;
  for (int j=1;j<=c.length();j++) {
    _data[c[j]]+=1;
  }
}











