#include "mvctsdistribution.h"
#include "myio.h"
//
//
//
dvec dirichlet::sample() 
{
	dvec x(np());
    double sum=0.0;
    for (int i =1;i<=np();i++) {
		x[i]=g.rgamma(par[i-1],1.0);
		sum+=x[i];
    }
	x/=sum;
	return x;
}

//
//
//
double dirichlet::log_pdf(const dvec &x)
{
	if (x.length()!=np()) 
		error e("wrong number of parameters in dirichlet log_pdf");
	
	double lx=0.0;
	for (int i=1;i<=np();i++) 
		lx += par[i-1]*log(x[i]);
	lx-=log_D(par,np());  // should make this a member variable to speed things up.
	return lx;
}
//
//
// Functions for the distributions
//
mvctsdistribution *mvscan(istream &in)
{
	char name[40];
	
    if (!nextname(in,name)) {
		cerr << "looking for " << name ; 
		cerr << "we need a multivariate continuous distribution";
		exit(0);
	}
	if (strcmp("undefined",name)==0) {
		mvundefctsdist *x = new mvundefctsdist;
		return x;
	} 
	dvec par = get_doublesfrombrackets(in);
	if (strcmp("constant",name)==0) {
		mvconstant *x = new mvconstant(par);
		return x;
	} else if (strcmp("dirichlet",name)==0) {
		dirichlet *x = new dirichlet(par);
		return x;
	} else {
		cerr << "this distribution not recognised\n";
		exit(0);
	}
}
//
//
//
mvctsdistribution *mvparscan(istream &in, char *namestring,char default_val[])
{
    if (findstart(in,namestring)) {
		return mvscan(in);
    } else {
		ofstream o("tmpfile");
		o<< default_val<< endl;
		cerr << namestring 
			<< " not found in parameter file, using "
			<< default_val << endl;
			ifstream in("tmpfile");
		return mvscan(in);
	}
}
//
//
//
ostream &operator<<(ostream &o, const mvctsdistribution &x)
{
	return x.print(o);
}

