#include "mvdrtdistribution.h"

#include "mvctsdistribution.h"

#include "myio.h"

#include "util.h"

//

//

// Functions for the distributions

//

mvdrtdistribution *mvdscan(istream &in)

{

	char name[40];

	

    if (!nextname(in,name)) {

		cerr << "looking for " << name ; 

		cerr << "we need a multivariate continuous distribution";

		exit(0);

	}

	if (strcmp("undefined",name)==0) {

		mvundefdrtdist *x = new mvundefdrtdist;

		return x;

	} 

	dvec par = get_doublesfrombrackets(in);

	if (strcmp("constant",name)==0) {

		mvdrtconstant *x = new mvdrtconstant(par);

		return x;

	} else if (strcmp("mndirichlet",name)==0) {

		mndirichlet *x = new mndirichlet(par);

		return x;

	} else {

		cerr << "this distribution not recognised\n";

		exit(0);

	}

}

//

//

//

mvdrtdistribution *mvdparscan(istream &in, char *namestring,char default_val[])

{

    if (findstart(in,namestring)) {

	  return mvdscan(in);

    } else {

		ofstream o("tmpfile");

		o<< default_val<< endl;

		cerr << namestring 

			<< " not found in parameter file, using "

			<< default_val << endl;

		ifstream in("tmpfile");

		return mvdscan(in);

	}

}

//

//  Multinomial dirichlet functions

//

double mndirichlet::log_pdf(const ivec &x)

{

	int i,tot=0;



	dvec ax(x.length());

	double temp = -log_D(alpha(),n());

	for (i=1;i<=x.length();i++) {

		tot+=x[i];

		ax[i]= alpha(i)+(double)x[i];

		temp -= lfactrl(x[i]);

	}



	temp += lfactrl(tot)+log_D(ax.data(),n());

	return temp;

}

//

//  

//

ivec mndirichlet::sample()

{

	dirichlet d(alpha(),n(),g);

	dvec x = d.sample();

	double temp = -log_D(alpha(),n());

	return rmultinomial(temp,n(),g);

}

//

//  Ewens distribution

//

double Ewens::log_pdf(const ivec &x)

{

	return 0.0;

}

//

ivec Ewens::sample() 

{

	ivec x(n());

	dvec a(n()+1);

	x[1]=1;

	a[1]=1.0;

	a[2]=theta();

	int k=2;

	for (int i=2;i<=n();i++) {

		x[i]=gen_from_p(a,k);

		if (x[i]<=k-1) a[x[i]]+=1.0;

		else {

			k+=1;

			a[k] = theta();

		}

	}

	ivec r(x,k-1);

	return r;

}



ivec rmultinomial(const dvec &p,int n,rng &r)

{

	ivec x(p.length());

	double prob=p[1];

	x[1]=rbinom(prob,n,r);

	double sum=1.-prob;

	n-=x[1];

	for (int i=2;i<p.length();i++,n>0) {

		double prob = p[i]/sum;

		x[i]=rbinom(prob,n,r);

		n -=x[i];

		sum -= prob;

	}

	x[p.length()]=n;

	return x;

}



ivec rmultinomialdirichlet(const dvec &a,int n,rng &r)

{

	dirichlet d(a,r);

	dvec p = d.sample();

	return rmultinomial(p,2,r);

}



ostream &operator<<(ostream &o, const mvdrtdistribution &x)

{

	return x.print(o);

}

