/* version1.0 (c) Copyright 2000 by Ian Wilson
   Written by Ian Wilson.
   Permission is granted to copy and use this program provided no fee is
   charged for it and provided that this copyright notice is not removed. */
#include <iostream>
#include <vector>
#include <strstream>
#include <ctype.h>
#include <string>
#include "util.h"
#include "myio.h"

vector <int> readintvector(istream &in,bool usechar, char until)
{
  vector <int> data;
  int x,ch;
	
  for (;;) {
    ch = skipblank(in,until);
    if ((isdigit(ch))||(ch=='-')||(ch=='N')||(usechar&&isalpha(ch))) {
      if (ch=='N') { // check for NA
	int ch2 = in.get();
	if (ch2=='A') 	x = -1;
	else {
	  if (!usechar) error e("got 'N' in int matrix"); 
	  in.putback(ch2);
	}
      } else {
	in.putback(ch);
	in >> x;
      }			
      data.push_back(x);
    }
    else break;
  }
  return data;
}
//
vector <int> readcharintvector(istream &in, char until)
{
	vector <int> data;
	int x,ch;
	
    for (;;) {
		ch = skipblank(in,until);
		if ((isdigit(ch))||isalpha(ch)||(ch=='-')) {
			in.putback(ch);
			in >> x;
			data.push_back(x);
		}
		else break;
    }
	return data;
}
//
// read doubles from a file (or keyboard) 
// until the first non number
//
vector <double> readdoublevector(istream &in, char until)
{	
	vector <double> data;
	int ch;
	double x;
	for (;;) {
		ch = skipblank(in,until);
		if (isdigit(ch)||ch=='-') {
			in.putback(ch);		
			in >> x;
			data. push_back(x);
		}
		else break;
    }	
	return data;
}
//
//

//
//   skip the rest of the line - used to 
//   enable comments at the start of lines
//
int skipline(istream &in)
/*  a little function to read to the end of a line 
    to remove comments !!                            */
{
	char tmp[2000];
	if (in.getline(tmp,1999)==NULL) {
		return EOF;	
	}
	return '\n';
}
//
//  skip spaces including line breaks
//
int skipspace(istream &in)
{
	static int ch;

	for (;;) {
		ch=in.get();
		if (ch=='#') ch = skipline(in);
		if (!isspace(ch)) return (ch);
	}
}
//
//  
//
int skipblank(istream &in, char until)
{
  static int ch;
  
  for (;;) {
    ch = in.get();
    if (ch=='#') ch = skipline(in);
    if (ch==until) return (ch);
    if (!isspace(ch)) return (ch);
  }
}
//
//  read a bool from an input stream
//
bool nextbool(istream &in)
{
    int ch = skipspace(in);
	if (ch=='T'||ch==1||ch=='t') return true;
	else if (ch=='F'||ch==0||ch=='f') return false;
	else error e("expecting a TRUE/FALSE value (T,t,1) or (F,f,0)");
	return false;
}
//
//  read a bool from an input stream
//
string nextstring(istream &in)
{
  string s;
  in >> s;
  return s;
}
// read an integer from a stream - separated from 
// the parameter name by a colon
int nextint(istream &in)
{
    int ch = skipspace(in);
 //   if (ch!=':') error e("error integer should be preceded by a colon");
	if (!isdigit(ch)||ch=='-') 
		error e("expecting an integer\n");
	in.putback(ch);

    in >> ch;
	
    return ch;
}
char nextchar(istream &in)
{
    int ch = skipspace(in);
	if (!isalpha(ch)) error e("expecting a character\n");
    return ch;
}
//
//  search the file for the first 
// time that pattern appears, 1 if a success 0 otherwise  
//
int findstart(istream &in,char *pattern)
{
  int c;
  char *p = pattern,comment='#';

  if (in.eof()) in.clear();     // if at end of file clear error
  in.seekg(0);				  // get back to start of file
  while ((c = in.get()) != EOF) {  
    if (*p == c) p++;
    else {
      if (comment == c) skipline(in);
      p = pattern;
    }
    if (!*p) return 1;
  }
  return 0;
}
//
//
//
bool findparametername(istream &in, const char *pattern)
{
	int c;
  const char *p = pattern,comment='#';

  if (in.eof()) in.clear();     // if at end of file clear error
  in.seekg(0);				  // get back to start of file
  while ((c = in.get()) != EOF) {  
    if (*p == c) p++;
    else {
      if (comment == c) skipline(in);
      p = pattern;
    }
    if (!*p) {
		if ((c=in.get()) == ':') return true;
		p=pattern;
	}
  }
  return false;
}
//
//  reads a character array from a parameter 
//  file - should really use a string object
//
//
// this should be depreciated
int nextname(istream &in, char *filename)
/*  filename must be at least 40 characters long */
{
    int ch,i;
    ch= skipspace(in);
    if (ch==':') ch= skipspace(in);
    for (i=0;i<40;i++) {
		if ((isspace(ch))||ch=='('||ch==EOF) {
			in.putback(ch);
			filename[i]='\0';
			return 1;
		}
		filename[i]=(char)ch;
		ch = in.get();
    }
    return 0;
}

/*********************************************************************/
double nextdouble(istream &in)
{
    skipspace(in);
	double tmp;
    in >> tmp ;
    return tmp;
}

/***************************************************************************/
/*    Now for scanning of parameter files                                  */
/***************************************************************************/
bool scan(istream &in, const char *namestring,bool default_val)
{
    if (findparametername(in,namestring)) {
		 return nextbool(in);
    } else {
		cout << namestring 
		     << " not found in parameter file, using "
		     << default_val << endl;
		return default_val;
    }
}
//

int scan(istream &in, char *namestring,int default_val)
{
    if (findparametername(in,namestring)) {
		 return nextint(in);
    } else {
		cout << namestring 
		     << " not found in parameter file, using "
		     << default_val << endl;
		return default_val;
    }
}

char scan(istream &in, char *namestring, char default_val)
{
  if (findparametername(in,namestring)) {
    return nextchar(in);
    } else {
      cout << namestring 
	   << " not found in parameter file, using "
	   << default_val << endl;
      return default_val;
    }
}
// now for floats and 



/***************************************************************************/
char *namescan(istream &in, char *namestring, char *default_val)
{
    char *temp = new char[100];
	
    if (findstart(in,namestring)) {
	nextname(in,temp);
    } else if (default_val!=NULL){
	cout << namestring 
	     << " not found: using default "
	     << default_val;
	istrstream in2(default_val);
	//strcpy(temp,default_val);
	nextname(in2,temp);
    } else {
	temp=NULL;
    }
    return temp;
}
/***************************************************************************/
/*int *intvector_scan(FILE *input, char *namestring,int *default_val)
  {
  int *tmp,i;
  if (findstart(input,namestring)) {
  return readintegerline(input);
  } else {
  if (default_val==NULL) {
  //	printf("%s not found in parameter file, using NULL\n",namestring);
			tmp=NULL;
		} else {
			tmp=(int*)MALLOC((default_val[0]+1)*sizeof(int));
			if (!tmp) myerror("error allocating intvector");
			for (i=0;i<=default_val[0];i++) tmp[i]=default_val[i];
		}
	}
	return tmp;	
}*/
/***************************************************************************/
/*char char_scan(FILE *input, char *namestring,char default_val)
{
    if (findstart(input,namestring)) {
		 return (char)nextint(input);
    } else {
		printf("%s not found in parameter file, using %c\n"
		,namestring,default_val);
			return default_val;
	}
}

/***************************************************************************/
double scan(istream &in, char *namestring,double default_val)
{
    if (findstart(in,namestring)) {
      return nextdouble(in);
    } else {
      cerr << namestring << "not found in parameter file, using " 
	   << default_val << endl;
      return default_val;
    }
}

/*
double double_scan_b(FILE *input, char *namestring,double default_val,volume vol)
{
    if (findstart(input,namestring)) {
		 return nextdouble(input);
    } else {
		if (vol==loud)
		printf("%s not found in parameter file, using %g\n"
		,namestring,default_val);
		return default_val;
	}
}
double *doublevector_scan(FILE *input, char *namestring,double *default_val, int *len)
{
	int i;
	double *tmp;
    if (findstart(input,namestring)) {
		 tmp=readdoubleline(input,len);
    } else {
		if (default_val==NULL||(*len)==0) {
			printf("%s not found in parameter file, using NULL\n",namestring);
			tmp=NULL;
			*len=0;
		} else {
			tmp=nrdvector(1,*len);
			for (i=1;i<=*len;i++) tmp[i]=default_val[i];
			printf("%s not found in parameter file, using\n");
			write_dvector(stdout," ",default_val,1,*len);
		}

	}		
	return tmp;
}*/

/***************************************************************************/
/*double *sequence_scan(FILE *input, char *namestring,int *n, char *default_val)
{
	double *temp,d[3];
	float fd[3];
	int i;

	if (findstart(input,namestring)) {
		fscanf(input,": %f %f %f",fd,fd+1,fd+2);
		for (i=0;i<3;i++) d[i]=(double)fd[i];
	} else {
		printf("unable to find sequence %s: using default %s\n",namestring,default_val);
		sscanf(default_val,"%f %f %f",fd,fd+1,fd+2);
		for (i=0;i<3;i++) d[i]=(double)fd[i];
    }
	if (fabs(d[1]-d[0])<d[2]) *n=1;
	else *n=(int)((d[1]+d[2]-d[0])/d[2]+0.5);
	temp=nrdvector(1,*n);
	for (i=0;i<*n;i++) temp[i+1]=d[0]+(double)i*d[2];
	return temp;
}*/
/************************************************************************/
//int **readraggedintegerarray(char *filename,int *rows)
/*  given a file this reads to the end of the file
 *  and returns the number of rows - the length oif each row is given by
 *  temp[i][0]                                                 */
/*{
    FILE *in;
    int i,j,rowstart,rowcount,*data,**toret,count=0,vecsize;
    static char ch;

    vecsize=FIRSTSIZE; //there is a blank so I can start at 1
	in=openinputfile(filename);
	*rows=0;
	
    data = (int*)MALLOC((vecsize+1)*sizeof(int));
    if (!data) nrerror("error allocating memory in readintegermatrix");

	rowstart=0;

	for (;;) {   //rows
		rowcount=0;
		for (;;) {  // columns  
			ch = (char)skipblank(in);
			if (isdigit(ch)||ch=='-') {
				ungetc(ch,in);
				if (count==vecsize) {
					data = (int*) REALLOC(data,(vecsize+1000+1)*sizeof(int));
					if (!data) nrerror("error reallocating data");
					vecsize +=1000;
				}
				fscanf(in,"%d",&(data[++count]));
				rowcount++;
			} else break;
		}
		if (rowcount>0) {
			data[rowstart]=rowcount;
			rowstart+=rowcount+1;
			count++;
			*rows+=1;
		}
		
		if (ch==EOF||ch==-1) break;
	}
    // remake the ragged array  
	toret=(int **)MALLOC((*rows+1)*sizeof(int *));
	if (!toret) myerror("error allocing memory for ragged integer array");
	count=0;
	for (i=1;i<=*rows;i++) {
		rowcount=data[count++];
		toret[i]=(int*)MALLOC((rowcount+1)*sizeof(int));
			toret[i][0]=rowcount;
			if (!toret[i]) myerror("error allocing memory for ragged integer array");
		for (j=1;j<=rowcount;j++)
			toret[i][j]=data[count++];

	}
	FREE(data);		

	return toret;
}*/
/************************************************************************/
//double **readraggeddoublearray(char *filename,int *rows)
/*  given a file this reads to the end of the file
 *  and returns the number of rows - the length oif each row is given by
 *  temp[i][0]                                                 */
/*{
    FILE *in;
    int i,j,rowstart,rowcount,count=0,vecsize;
	double *data,**toret;
    static char ch;
	float temf;

    vecsize=FIRSTSIZE; //there is a blank so I can start at 1
	in=openinputfile(filename);
	*rows=0;
	
    data = (double*)MALLOC((vecsize+1)*sizeof(double));
    if (!data) nrerror("error allocating memory in readintegermatrix");

	rowstart=0;

	for (;;) {   //rows
		rowcount=0;
		for (;;) {  // columns  
			ch = (char)skipblank(in);
			if (isdigit(ch)||ch=='-') {
				ungetc(ch,in);
				if (count==vecsize) {
					data = (double*) REALLOC(data,(vecsize+1000+1)*sizeof(double));
					if (!data) nrerror("error reallocating data");
					vecsize +=1000;
				}
				fscanf(in,"%g",&temf);
				data[++count]=(double)temf;
				rowcount++;
			} else break;
		}
		if (rowcount>0) {
			data[rowstart]=(double)rowcount;
			rowstart+=rowcount+1;
			count++;
			*rows+=1;
		}
		
		if (ch==EOF||ch==-1) break;
	}
    // remake the ragged array  
	toret=(double **)MALLOC((*rows+1)*sizeof(double *));
	if (!toret) myerror("error allocing memory for ragged integer array");
	count=0;
	for (i=1;i<=*rows;i++) {
		rowcount=(int)(data[count++]+0.5);
		toret[i]=(double*)MALLOC((rowcount+1)*sizeof(double));
			toret[i][0]=rowcount;
			if (!toret[i]) myerror("error allocing memory for ragged integer array");
		for (j=1;j<=rowcount;j++)
			toret[i][j]=data[count++];

	}
	FREE(data);		
	return toret;
}*/
/***************************************************************************/

/*************************************************************************/
/*  function to get a string from keyboard - skipping
	all blanks in the input - suitable for getting file
	names for example.  If nothing is entered then 
	def_val is used as a default                   */
/*************************************************************************/
/*char *getstringfromkeyboard(char *message, char *def_val)
{
	char *tmp;
	int i;
	int ch;

	tmp = (char*)malloc(200*sizeof(char));
	if (!tmp) myerror("error allocating memory in getstringfromkeyboard");
	cout << "please enter "
		<< message 
		<< " or press <enter> for default ("
		<< def_val
		<< ")\n";
	for (i=0;;) {
		ch=skipblank(cin);
		if (ch=='\n') {
			tmp[i++]='\0';
			break;
		} else tmp[i++]=(char)ch;
	}
	if (i==1) strcpy(tmp,def_val);
	return tmp;
}*/
/*************************************************************************/
/* A file with a collection of utilities for input and output            */
/*************************************************************************/
/*char *allocatestring(char *tocopy)
{
	int len;
	char *ret;
	
	if (tocopy !=NULL) {
		len=strlen(tocopy);
		ret=(char *)MALLOC((len+1)*sizeof(char));    
		if (!ret) myerror("error allocating string");
		strcpy(ret,tocopy);
		return ret;
	} else return NULL;
}
*/

/*************************************************************************/
/*int *nextints(FILE *in,int len)
{
    int ch,*retvec,i;
    retvec=nrivector(1,len);
    ch = skipspace(in);
    if (ch!=':')
	nrerror("error integer should be preceded by a colon");
    for (i=1;i<=len;i++)
    	fscanf(in,"%d",retvec+i);
    return retvec;
}*/
/*************************************************************************/
//void get_counter_filename(char *name, char *base, int count)
//{
  //  sprintf(name,"%s.%d",base,count);
//}
/****************************************************************/
/*char *readfromquotes(FILE *in,int *len)
{
	int count=0,ch,vecsize=FIRSTSIZE;
	char *data;
	
	ch = skipspace(in);
	if (ch!=39) {
		printf("error - should be a quote - ' is %c\n",ch);
		exit(0);
	}
	data = (char*)MALLOC(FIRSTSIZE*sizeof(char));
	if (!data) nrerror("error allocating memory in readfromquotes");
	for (count=0;;count++) {
		if (count==vecsize) {
			data = (char *)REALLOC(data,(vecsize+1000)*sizeof(char));
			if (!data) nrerror("error reallocating data in readfromquotes");
			vecsize +=1000;
		}
		ch = fgetc(in);
		if (ch==39) {
			data[count]='\0';
			break;
		}
		else data[count]=(char)ch;
		
	}
	data = (char *)REALLOC(data,(count+1)*sizeof(char));
    if (!data) nrerror("error reallocating data");
    *len = count+1;
	return data;
}*/
/*double *readdoubleline(FILE *in, int *len)
{
    int count=0,vecsize=FIRSTSIZE;
    int ch;
    double *data;
    
	ch = skipspace(in);
	if (ch!=':') {
		ungetc(ch,in);
	}

    data = (double*)MALLOC(FIRSTSIZE*sizeof(double));
    if (!data) nrerror("error allocating memory in readintegerline");
    for (;;) {
		ch = skipspace(in);
		if (isdigit(ch)||ch=='-') {
			ungetc(ch,in);
			if (count==vecsize) {
				data = (double *)REALLOC(data,(vecsize+1000)*sizeof(double));
				if (!data) nrerror("error reallocating data");
				vecsize +=1000;
			}
			fscanf(in,"%lf",&(data[++count]));
		}
		else break;
    }
    data = (double *)REALLOC(data,(count+1)*sizeof(double));
    if (!data) nrerror("error reallocating data");
    *len = count;
    return data;
*/
/************************************************************************/
/*void write_raggedintegerarray(FILE *out, int **rag, int rows, int *cols)
{
    int i,j;
    for (i=1;i<=rows;i++) {
		for (j=1;j<=cols[i];j++)
			fprintf(out,"%d ",rag[i][j]);
		fprintf(out,"\n");
    }
    return;
}*/
/************************************************************************/
/*void write_raggeddoublearray(FILE *out, double **rag, int rows, int *cols)
{
    int i,j;
    for (i=1;i<=rows;i++) {
		for (j=1;j<=cols[i];j++)
			fprintf(out,"%g ",rag[i][j]);
		fprintf(out,"\n");
    }
    return;
}*/
/************************************************************************/
/*void write_dvector(FILE *out,char *sep, double *vec, int sr, int er)
{
    int i;
    for (i=sr;i<er;i++) {
			fprintf(out,"%g%s",vec[i],sep);    
	}
		fprintf(out,"%g\n",vec[er],sep); 
    return;
}*/
/************************************************************************/
/*void write_ivector(FILE *out, char *sep, int *vec, int sr, int er)
{
    int i;
    for (i=sr;i<er;i++) {
			fprintf(out,"%d%s",vec[i],sep);    
	}
	fprintf(out,"%d\n",vec[er]);
    return;
}*/



/*char **read_characterarray(FILE *in,int *len)
{
	char **toret,space[]=" \t\n";
	int maxcount=10000;
	*len=1;


	toret= (char **)MALLOC(maxcount*sizeof(char*));
	if (!toret) myerror("error allocating memory in read_characterarray");
	toret[0] = (char *)MALLOC((1001)*sizeof(char));
	if (!toret[0]) myerror("error alocating memory");

	if (fgets(toret[0],1000,in)==NULL) myerror("error reading placenames");
	toret[0]=(char *)REALLOC(toret[0],(1+strlen(toret[0]))*sizeof(char));
	if (!toret[0]) myerror("error reallocating memory");

	toret[*len]=strtok(toret[0],space);
    while( toret[*len] != NULL ){
		*len+=1;
		toret[*len] = strtok(NULL,space);
	}
	*len-=1;
	
	toret = (char **)REALLOC(toret,(*len+1)*sizeof(char *));
	if (!toret) myerror("error reallocating memory in read_characterarray");

	return toret;
}*/
/************************************************************************/
/*char *readstring(FILE *in)
{
	char *temp;
	temp = (char *)MALLOC((101)*sizeof(char));
	if (!temp) myerror("error allocating memory");
	fscanf(in,"%s\n",temp);
	temp=(char *)REALLOC(temp,(1+strlen(temp))*sizeof(char));
	if (!temp) myerror("error reallocating memory");
	return temp;
}*/


/************************************************************************/
/* int **readintegermatrix(FILE *in,int *rows, int *columns)
//  given a file this reads until it gets a non integer    
{
    int i,j,*data,**toret,count=0,vecsize;
    static char ch;

    vecsize=FIRSTSIZE; // there is a blank so I can start at 1 
	data = (int*)MALLOC((vecsize+1)*sizeof(int));
    if (!data) nrerror("error allocating memory in readintegermatrix");

	ch=(char)skipspace(in);
	ungetc(ch,in);
    
    // first find out how many columns  
    for (;;) {
		ch = (char)skipblank(in);
		if (isdigit(ch)||ch=='-') {
			ungetc(ch,in);
			if (count==vecsize) {
				data = (int*) REALLOC(data,(vecsize+1000+1)*sizeof(int));
				if (!data) nrerror("error reallocating data");
				vecsize +=1000;
			}
			fscanf(in,"%d",&(data[++count]));
		} else if ((ch =='\n')||(ch == EOF)) {
			*columns = count;
			break;
		}
		else
	    nrerror("This file should just contain integers");
    }
    // now do the rest of the file  
    for (;;) {
		ch = (char)skipspace(in);
		if (isdigit(ch)||ch=='-') {
			ungetc(ch,in);
			if (count==vecsize) {
				data = (int*) REALLOC(data,(vecsize+1000+1)*sizeof(int));
				if (!data) nrerror("error reallocating data");
				vecsize +=1000;
			}
			fscanf(in,"%d",&(data[++count]));
		} else {
			ungetc(ch,in);
			break;
		}
    }
    
    if (count%(*columns)!=0) {
        printf("WARNING the file does not *fill* out the matrix count %d\n",
		count);
        *rows = count/(*columns) +1;
    }
    else *rows = count/(*columns);

    toret = nrimatrix(1,*rows,1,*columns);
    count =1;
    for (i=1;i<=*rows;i++) 
	for (j=1;j<=*columns;j++)
	    toret[i][j]=data[count++];
    FREE(data);
    return toret;
}
*/
//
//
//
vector <int> readcharintmat(istream &in,int &cols, bool usechar)
//  this reads a mixture of integer
{   
    // first find out how many columns  
	{
		vector <int> tmp;
		tmp = readintvector(in,usechar,'\n');
		cols=tmp.size();
	}		
	if (in.eof()) in.clear();     // if at end of file clear error
	in.seekg(0);
	vector <int> tmp;
	tmp = readintvector(in,usechar,EOF);
	return tmp;
}
//
//
//
vector <double> readdoublemat(istream &in,int &cols)
//  this reads a mixture of integer
{   
    // first find out how many columns  
	{
		vector <double> tmp;
		tmp = readdoublevector(in,'\n');
		cols=tmp.size();
	}		
	if (in.eof()) in.clear();     // if at end of file clear error
	in.seekg(0);
	vector <double> tmp;
	tmp = readdoublevector(in,EOF);
	return tmp;
}
//
//
//
