#include <cassert>
#include "random.h"

//
// The global random number generator
//
// acts as a default for all random number generation.
//
//
rng globalrandom;  
/******************************************************************/
/*  set up the  KISS random number generator                      */
/******************************************************************/
void rng::seed(unsigned int seedval)
{
     x=seedval;y=102;z=12;w=34535;
     x = x * 69069 + 1;
     y ^= y << 13;
     y ^= y >> 17;
     y ^= y << 5;
     k = (z >> 2) + (w >> 3) + (carry >> 2);
     m = w + w + z + carry;
     z = w;
     w = m;
     carry = k >> 30;
	 iset=0;
}
//
void rng::set(unsigned int *vals)
{
	k=vals[0];
	m=vals[1];
	x=vals[2];
	y=vals[3];
	z=vals[4];
	w=vals[5];
	carry=vals[6];
	r=vals[7];
	iset=0;
}
//
void rng::state(unsigned int *vals)
{
	vals[0]=k;
	vals[1]=m;
	vals[2]=x;
	vals[3]=y;
	vals[4]=z;
	vals[5]=w;
	vals[6]=carry;
	vals[7]=r;
}
//
//  The Marsalia KISS rng
//
unsigned int rng::nextui(void)
{
     x = x * 69069 + 1;
     y ^= y << 13;
     y ^= y >> 17;
     y ^= y << 5;
     k = (z >> 2) + (w >> 3) + (carry >> 2);
     m = w + w + z + carry;
     z = w;
     w = m;
     carry = k >> 30;
     return x+y+z;
}
//
/****************************************************************************************/
/*
 *  Mathlib : A C Library of Special Functions
 *  Copyright (C) 1998 Ross Ihaka
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  SYNOPSIS
 *
 *    #include "Mathlib.h"
 *    double sexp(void);
 *
 *  DESCRIPTION
 *
 *    Random variates from the standard exponential distribution.
 *
 *  REFERENCE
 *
 *    Ahrens, J.H. and Dieter, U. (1972).
 *    Computer methods for sampling from the exponential and
 *    normal distributions.
 *    Comm. ACM, 15, 873-882.
 */
/****************************************************************************************/
double rng::sexp(void)
{
    /* q[k-1] = sum(aLOG(2.0)**k/k!) k=1,..,n, */
    /* The highest n (here 8) is determined by q[n-1] = 1.0 */
    /* within standard precision */
    const double q[] =
    {
		0.6931471805599453,
			0.9333736875190459,
			0.9888777961838675,
			0.9984959252914960,
			0.9998292811061389,
			0.9999833164100727,
			0.9999985691438767,
			0.9999998906925558,
			0.9999999924734159,
			0.9999999995283275,
			0.9999999999728814,
			0.9999999999985598,
			0.9999999999999289,
			0.9999999999999968,
			0.9999999999999999,
			1.0000000000000000
    };
    double a, u, ustar, umin;
    int i;

    a = 0.0;
    u = next();
    for (;;) {
		u = u + u;
		if (u > 1.0)
			break;
		a = a + q[0];
    }
    u = u - 1.0;

    if (u <= q[0])
		return a + u;

    i = 0;
    ustar = next();
    umin = ustar;
    do {
		ustar = next();
		if (ustar < umin)
			umin = ustar;
		i = i + 1;
    }
    while (u > q[i]);
    return a + umin * q[0];
}
/****************************************************************/
const double a1 = 0.3333333;
const double a2 = -0.250003;
const double a3 = 0.2000062;
const double a4 = -0.1662921;
const double a5 = 0.1423657;
const double a6 = -0.1367177;
const double a7 = 0.1233795;
const double e1 = 1.0;
const double e2 = 0.4999897;
const double e3 = 0.166829;
const double e4 = 0.0407753;
const double e5 = 0.010293;
const double q1 = 0.04166669;
const double q2 = 0.02083148;
const double q3 = 0.00801191;
const double q4 = 0.00144121;
const double q5 = -7.388e-5;
const double q6 = 2.4511e-4;
const double q7 = 2.424e-4;
const double sqrt32 = 5.656854;
static double aa = 0.;
static double aaa = 0.;
double rng::rgamma(double a, double scale)
{
#define repeat for(;;)
/* Taken from R */

	static double b, c, d, e, p, q, r, s, t, u, v, w, x;
	static double q0, s2, si;
	double ret_val;
	
	if (a < 1.0) {
		/* alternate method for parameters a below 1 */
		/* 0.36787944117144232159 = exp(-1) */
		aa = 0.0;
		b = 1.0 + 0.36787944117144232159 * a;
		repeat {
			p = b * next();
			if (p >= 1.0) {
				ret_val = -log((b - p) / a);
				if (sexp() >= (1.0 - a) * log(ret_val))
					break;
			} else {
				ret_val = exp(log(p) / a);
				if (sexp() >= ret_val)
					break;
			}
		}
		return scale * ret_val;
	}
	/* Step 1: Recalculations of s2, s, d if a has changed */
	if (a != aa) {
		aa = a;
		s2 = a - 0.5;
		s = sqrt(s2);
		d = sqrt32 - s * 12.0;
	}
	/* Step 2: t = standard normal deviate, */
	/* x = (s,1/2)-normal deviate. */
	/* immediate acceptance (i) */
	
	t = normal();
	x = s + 0.5 * t;
	ret_val = x * x;
	if (t >= 0.0)
		return scale * ret_val;
	
	/* Step 3: u = 0,1 - uniform sample. squeeze acceptance (s) */
	u = next();
	if (d * u <= t * t * t) {
		return scale * ret_val;
	}
	/* Step 4: recalculations of q0, b, si, c if necessary */
	
	if (a != aaa) {
		aaa = a;
		r = 1.0 / a;
		q0 = ((((((q7 * r + q6) * r + q5) * r + q4)
			* r + q3) * r + q2) * r + q1) * r;
		
		/* Approximation depending on size of parameter a */
		/* The constants in the expressions for b, si and */
		/* c were established by numerical experiments */
		
		if (a <= 3.686) {
			b = 0.463 + s + 0.178 * s2;
			si = 1.235;
			c = 0.195 / s - 0.079 + 0.16 * s;
		} else if (a <= 13.022) {
			b = 1.654 + 0.0076 * s2;
			si = 1.68 / s + 0.275;
			c = 0.062 / s + 0.024;
		} else {
			b = 1.77;
			si = 0.75;
			c = 0.1515 / s;
		}
	}
	/* Step 5: no quotient test if x not positive */
	
	if (x > 0.0) {
		/* Step 6: calculation of v and quotient q */
		v = t / (s + s);
		if (fabs(v) <= 0.25)
			q = q0 + 0.5 * t * t * ((((((a7 * v + a6)
			* v + a5) * v + a4) * v + a3)
			* v + a2) * v + a1) * v;
		else
			q = q0 - s * t + 0.25 * t * t + (s2 + s2)
			* log(1.0 + v);
		
		
		/* Step 7: quotient acceptance (q) */
		
		if (log(1.0 - u) <= q)
			return scale * ret_val;
	}
	/* Step 8: e = standard exponential deviate */
	/* u= 0,1 -uniform deviate */
	/* t=(b,si)-double exponential (laplace) sample */
	
	repeat {
		e = sexp();
		u = next();
		u = u + u - 1.0;
		if (u < 0.0)
			t = b - si * e;
		else
			t = b + si * e;
		/* Step  9:  rejection if t < tau(1) = -0.71874483771719 */
		if (t >= -0.71874483771719) {
			/* Step 10:  calculation of v and quotient q */
			v = t / (s + s);
			if (fabs(v) <= 0.25)
				q = q0 + 0.5 * t * t * ((((((a7 * v + a6)
				* v + a5) * v + a4) * v + a3)
				* v + a2) * v + a1) * v;
			else
				q = q0 - s * t + 0.25 * t * t + (s2 + s2)
				* log(1.0 + v);
			/* Step 11:  hat acceptance (h) */
			/* (if q not positive go to step 8) */
			if (q > 0.0) {
				if (q <= 0.5)
					w = ((((e5 * q + e4) * q + e3)
					* q + e2) * q + e1) * q;
				else
					w = exp(q) - 1.0;
				/* if t is rejected */
				/* sample again at step 8 */
				if (c * fabs(u) <= w * exp(e - 0.5 * t * t))
					break;
			}
		}
	}
	x = s + 0.5 * t;
	return scale * x * x;
}
/*************************************************************************/
double rng::normal(void)
{
	double  fac,rsq,v1,v2;
	
	if (iset == 0) {
		do {
			v1 = 2.0*next( )-1.0;
			v2 = 2.0*next( )-1.0;
			rsq = v1*v1+v2*v2;
		} while (rsq >= 1.0 || rsq == 0);
		fac = sqrt(-2.0*log(rsq)/rsq);
		gset = v1 * fac;
		iset = 1;
		return v2*fac;
	} else {
		iset = 0;
		return  gset;
	}
}
//
// some global utility functions
//
void sample2int(int wh[2],int from, int to, rng &g)
{
	double len;
	len=(double)(to-from+1);
	wh[0] = from + (int)(g.next()*len);	
wh[1] = from + (int)(g.next()*(len-1.));
	if (wh[1]>=wh[0]) wh[1]++;
	return;
}
//
//
//
int  gen_from_p(dvec &p, rng &r)
{
  dvec cprob(p.length());
  double sum = p.sum();
  assert(sum>0.0);
  cprob[1]=p[1]/sum;
  for (int i=2;i<=p.length();i++) cprob[i]=cprob[i-1]+p[i]/sum;
  return gen_from_cp(cprob);
}
//
//
//
int  gen_from_p(dvec &p, int n, rng &r)
{
  dvec cprob(n);
  double sum = p.sum(n);
  assert(sum>0.0);
  cprob[1]=p[1]/sum;
  for (int i=2;i<=n;i++) cprob[i]=cprob[i-1]+p[i]/sum;
  return gen_from_cp(cprob);
}
/*******************************************************************/
int gen_from_cp(dvec &cp, rng &r)
// a routine to pick a number from 1 to n based on cumulative
// probabilities in p    
{
	double prob = r();
	int where = (int)prob*(cp.length()) + 1;
	for (;;) {
		if  (prob <= cp [where]) {
			if (where==1) return 1;
			if (prob > cp[where-1]) return where;
			else where--;
		} else {
			where++;
			if (where>=cp.length()) return cp.length();
		}
	}
}
//
//   Binomial random variable
//
/**************************************************************/
int rbinom(double  pp,int n, rng &r)
{
    int j;
    static int nold=(-1);
    double am,em,g,angle,p,bnl,sq,t,y;
    static double pold=(-1.0),pc,plog,pclog,en,oldg;

    p = (pp<=0.5 ? pp: 1.0-pp);

    am = n*p;
    if (n<25) {
        bnl=0.0;
        for (j=1;j<=n;j++)
            if (r() < p) ++bnl;
    }
    else if (am < 1.0) {
        g = exp(-am);
        t=1.0;
        for (j=0;j<=n;j++)
        {
            t *= r();
            if (t<g) break;
        }
        bnl = (j<=n ? j : n);
    }
    else {
        if (n!=nold) {
            en=n;
            oldg=lgamma(en+1.0);
            nold=n;
        }
        if (p!=pold) {
            pc = 1.0-p;
            plog=log(p);
            pclog=log(pc);
            pold=p;
        }
        sq=sqrt(2.0*am*pc);
        do {
            do {
                angle = M_PI*r();
                y=tan(angle);
                em=sq*y+am;
            } while (em <0.0 || em >= (en + 1.0));
            em = floor(em);
            t=1.2*sq*(1.0+y*y)*exp(oldg-lgamma(em+1.0)
                -lgamma(en-em+1.0)+em*plog+(en-em)*pclog);
        } while (r() > t);
        bnl = em;
    }
    if (p!= pp) bnl=n-bnl;
    return int(bnl+0.5);
}
//
/****************************************************************/
int poidev(double xm, rng &r)
{
	static double sq,alxm,g,oldm=(-1.0);
	double em,t,y;

	if (xm < 12) {
		if (xm != oldm) {
			oldm = xm;
			g = exp(-xm);
		}
		em = -1;
		t = 1.0;
		do {
			++em;
			t*=r();
		} while (t > g);
	} else {
		if (xm!=oldm) {
			oldm = xm;
			sq = sqrt(2.0*xm);
			alxm = log(xm);
			g=xm*alxm-lgamma(xm+1.0);
		}
		do {
			do {
				y=tan(M_PI*r());
				em = sq*y+xm;
			} while (em < 0.0);
			em = floor(em);
			t=0.9*(1.0+y*y)*exp(em*alxm-lgamma(em+1.0)-g);
		} while (r()>t);
	}
	return int(em+0.5);
}



//
// log of the normal distribution p.d.f
//
double ldnorm(double x, double mu, double sigma) 
{
		double lx = (x-mu)/sigma;
		return LMY_SQRT_2PI - log(sigma) - 0.5*lx*lx;
}

