#ifndef RANDOM_H__
#define RANDOM_H__
#include <iostream>
#include <cmath>
#include "util.h"
#include "dvec.h"
using namespace::std;
/******************************************************************/
/*   Keep It Simple Stupid random number generator from George 
     Marsaglia's DIEHARD cdrom                                    */
//  I think that I should actually have more than one random number
//  generator and have a virtual base class rng - but not for now
//

class rng {
public:
	 double next() { 
		 return ((double)nextui()+0.5)/4294967296.0;
	 }
	 rng(unsigned int seedval=1) {seed(seedval);}
	 rng(unsigned int *vals) {set(vals);}
	 double operator()(void) {return next();} // call like a function
	 double normal(void);
	 double rgamma(double a, double scale);
	 int rint(int s,int f) {return int(s+next()*(f-s+0.5));}
	 double sexp(void);
	 void state(unsigned int *vals);
	 void set(unsigned int *vals);
	 void set(unsigned int seedval=1) {seed(seedval);}
 private:
	 unsigned int nextui();	 
	 void seed(unsigned int seedval);
	 unsigned int k,m,x,y,z,w,carry,r;
	 int iset;  // needed for normals generated by Box-Muller 
	 double gset;
};
extern rng globalrandom; 
// the default random number generator - if you do not pass a 
// random number generator then this one will be used.  
void sample2int(int wh[2],int from, int to, rng &r = globalrandom);
double ldnorm(double x, double mu, double sigma);
int  gen_from_p(dvec &p, rng &r = globalrandom);
int  gen_from_p(dvec &p,int n, rng &r = globalrandom);
int gen_from_cp(dvec &cp,rng &r = globalrandom);
int rbinom(double  pp,int n, rng &r=globalrandom);
int poidev(double xm, rng &r=globalrandom);
#endif
